'use strict';

(function (global) {

    var DropUploader = React.createClass({
        displayName: 'DropUploader',

        onDrop: function onDrop(files, event, sourceComponent) {
            var items, files;
            if (event.dataTransfer) {
                items = event.dataTransfer.items || [];
                files = event.dataTransfer.files;
            } else if (event.target) {
                files = event.target.files;
            }
            var contextNode = global.pydio.getContextHolder().getContextNode();
            UploaderModel.Store.getInstance().handleDropEventResults(items, files, contextNode);
        },

        onFolderPicked: function onFolderPicked(files) {
            var contextNode = global.pydio.getContextHolder().getContextNode();
            UploaderModel.Store.getInstance().handleFolderPickerResult(files, contextNode);
        },

        start: function start(e) {
            e.preventDefault();
            UploaderModel.Store.getInstance().processNext();
        },
        clear: function clear(e) {
            e.preventDefault();
            UploaderModel.Store.getInstance().clearAll();
        },
        toggleOptions: function toggleOptions(e) {
            e.preventDefault();
            var crtOptions = this.state && this.state.options ? this.state.options : false;
            this.setState({ options: !crtOptions });
        },

        openFilePicker: function openFilePicker(e) {
            e.preventDefault();
            this.refs.dropzone.open();
        },

        openFolderPicker: function openFolderPicker(e) {
            e.preventDefault();
            this.refs.dropzone.openFolderPicker();
        },

        render: function render() {

            var options = undefined;
            if (this.state && this.state.options) {
                options = React.createElement(UploadOptionsPane, { onDismiss: this.toggleOptions });
            }
            var folderButton = undefined,
                startButton = undefined;
            var e = global.document.createElement('input');
            e.setAttribute('type', 'file');
            if ('webkitdirectory' in e) {
                folderButton = React.createElement(ReactMUI.RaisedButton, { label: 'Pick Folder', onClick: this.openFolderPicker });
            }
            e = null;
            var configs = UploaderModel.Configs.getInstance();
            if (!configs.getOptionAsBool('DEFAULT_AUTO_START', 'upload_auto_send', true)) {
                startButton = React.createElement(ReactMUI.FlatButton, { label: 'Start', onClick: this.start, secondary: true });
            }
            return React.createElement(
                'div',
                { style: { position: 'relative' } },
                React.createElement(
                    'div',
                    { className: 'react-mui-context uploader-action-bar' },
                    React.createElement(ReactMUI.FlatButton, { style: { float: 'right' }, label: 'Options', onClick: this.toggleOptions }),
                    React.createElement(ReactMUI.RaisedButton, { secondary: true, label: 'Pick File', onClick: this.openFilePicker }),
                    folderButton,
                    startButton,
                    React.createElement(ReactMUI.FlatButton, { label: 'Clear List', onClick: this.clear })
                ),
                React.createElement(
                    PydioForm.FileDropZone,
                    {
                        ref: 'dropzone',
                        multiple: true,
                        enableFolders: true,
                        supportClick: false,
                        onDrop: this.onDrop,
                        onFolderPicked: this.onFolderPicked,
                        style: { width: '100%' }
                    },
                    React.createElement(TransfersList, null)
                ),
                options
            );
        }

    });

    var TransferFile = React.createClass({
        displayName: 'TransferFile',

        propTypes: {
            item: React.PropTypes.object.isRequired,
            className: React.PropTypes.string
        },

        componentDidMount: function componentDidMount() {
            this.props.item.observe('progress', (function (value) {
                this.setState({ progress: value });
            }).bind(this));
            this.props.item.observe('status', (function (value) {
                this.setState({ status: value });
            }).bind(this));
        },

        getInitialState: function getInitialState() {
            return {
                progress: this.props.item.getProgress(),
                status: this.props.item.getStatus()
            };
        },

        abortTransfer: function abortTransfer() {
            UploaderModel.Store.getInstance().stopOrRemoveItem(this.props.item);
        },

        render: function render() {
            var style = undefined;
            var messageIds = {
                "new": 433,
                "loading": 434,
                "loaded": 435,
                "error": 436
            };
            var statusMessage = this.props.item.getStatus();
            var stopButton = undefined;
            if (statusMessage === 'loading') {
                stopButton = React.createElement('span', { className: 'stop-button icon-stop', onClick: this.abortTransfer });
            } else {
                stopButton = React.createElement('span', { className: 'stop-button mdi mdi-close', onClick: this.abortTransfer });
            }
            if (statusMessage === 'error' && this.props.item.getErrorMessage()) {
                statusMessage = this.props.item.getErrorMessage();
            }
            if (global.pydio.MessageHash[messageIds[statusMessage]]) {
                statusMessage = global.pydio.MessageHash[messageIds[statusMessage]];
            }
            if (this.props.item.getRelativePath()) {
                var relativeMessage = React.createElement(
                    'span',
                    { className: 'path' },
                    this.props.item.getRelativePath()
                );
            }
            if (this.state && this.state.progress) {
                style = { width: this.state.progress + '%' };
            }
            return React.createElement(
                'div',
                { className: "file-row upload-" + this.props.item.getStatus() + " " + (this.props.className ? this.props.className : "") },
                React.createElement('span', { className: 'mdi mdi-file' }),
                ' ',
                this.props.item.getFile().name,
                relativeMessage,
                React.createElement(
                    'span',
                    { className: 'status' },
                    statusMessage
                ),
                stopButton,
                React.createElement('div', { className: 'uploader-pgbar', style: style })
            );
        }
    });

    var TransferFolder = React.createClass({
        displayName: 'TransferFolder',

        propTypes: {
            item: React.PropTypes.object.isRequired
        },

        render: function render() {
            var statusMessage = undefined;
            if (this.props.item.getStatus() === 'loaded') {
                statusMessage = 'Created';
            }
            return React.createElement(
                'div',
                { className: "folder-row upload-" + this.props.item.getStatus() + " " + (this.props.className ? this.props.className : "") },
                React.createElement('span', { className: 'mdi mdi-folder' }),
                ' ',
                this.props.item.getPath(),
                ' ',
                React.createElement(
                    'span',
                    { className: 'status' },
                    statusMessage
                )
            );
        }
    });

    var TransfersList = React.createClass({
        displayName: 'TransfersList',

        componentDidMount: function componentDidMount() {
            var store = UploaderModel.Store.getInstance();
            this._storeObserver = (function () {
                if (!this.isMounted()) return;
                this.setState({ items: store.getItems() });
            }).bind(this);
            store.observe("update", this._storeObserver);
            store.observe("auto_close", function () {
                pydio.UI.modal.dismiss();
            });
            this.setState({ items: store.getItems() });
        },

        componentWillUnmount: function componentWillUnmount() {
            if (this._storeObserver) {
                UploaderModel.Store.getInstance().stopObserving("update", this._storeObserver);
                UploaderModel.Store.getInstance().stopObserving("auto_close");
            }
        },

        renderSection: function renderSection(accumulator, items) {
            var title = arguments.length <= 2 || arguments[2] === undefined ? "" : arguments[2];
            var className = arguments.length <= 3 || arguments[3] === undefined ? "" : arguments[3];

            if (title && items.length) {
                accumulator.push(React.createElement(
                    'div',
                    { className: className + " header" },
                    title
                ));
            }
            items.sort(function (a, b) {
                var aType = a instanceof UploaderModel.FolderItem ? 'folder' : 'file';
                var bType = b instanceof UploaderModel.FolderItem ? 'folder' : 'file';
                if (aType === bType) {
                    return 0;
                } else {
                    return aType === 'folder' ? -1 : 1;
                }
            });
            items.forEach(function (f) {
                if (f instanceof UploaderModel.FolderItem) {
                    accumulator.push(React.createElement(TransferFolder, { key: f.getId(), item: f, className: className }));
                } else {
                    accumulator.push(React.createElement(TransferFile, { key: f.getId(), item: f, className: className }));
                }
            });
        },

        render: function render() {
            var items = [];
            if (this.state && this.state.items) {
                this.renderSection(items, this.state.items.processing, 'Processing', 'section-processing');
                this.renderSection(items, this.state.items.pending, 'Pending', 'section-pending');
                this.renderSection(items, this.state.items.processed, 'Processed', 'section-processed');
            }
            return React.createElement(
                'div',
                { id: 'upload_files_list', className: UploaderModel.Configs.getInstance().getOptionAsBool('UPLOAD_SHOW_PROCESSED', 'upload_show_processed', false) ? 'show-processed' : '' },
                items
            );
        }
    });

    var UploadOptionsPane = React.createClass({
        displayName: 'UploadOptionsPane',

        propTypes: {
            onDismiss: React.PropTypes.func.isRequired
        },

        getInitialState: function getInitialState() {

            var configs = UploaderModel.Configs.getInstance();
            return {
                configs: configs
            };
        },

        updateField: function updateField(fName, event) {
            if (fName === 'autostart') {
                var toggleStart = this.state.configs.getOptionAsBool('DEFAULT_AUTO_START', 'upload_auto_send', true);
                toggleStart = !toggleStart;
                this.state.configs.updateOption('upload_auto_send', toggleStart, true);
            } else if (fName === 'autoclose') {
                var toggleStart = this.state.configs.getOptionAsBool('DEFAULT_AUTO_CLOSE', 'upload_auto_close', true);
                toggleStart = !toggleStart;
                this.state.configs.updateOption('upload_auto_close', toggleStart, true);
            } else if (fName === 'existing') {
                this.state.configs.updateOption('upload_existing', event.target.getSelectedValue());
            } else if (fName === 'show_processed') {
                var toggleShowProcessed = this.state.configs.getOptionAsBool('UPLOAD_SHOW_PROCESSED', 'upload_show_processed', false);
                toggleShowProcessed = !toggleShowProcessed;
                this.state.configs.updateOption('upload_show_processed', toggleShowProcessed, true);
            }
            this.setState({ random: Math.random() });
        },

        radioChange: function radioChange(e, newValue) {
            this.state.configs.updateOption('upload_existing', newValue);
            this.setState({ random: Math.random() });
        },

        render: function render() {

            var maxUpload = this.state.configs.getOption('UPLOAD_MAX_SIZE');
            var maxUploadMessage = MessageHash[282] + ': ' + PathUtils.roundFileSize(maxUpload, '');
            var toggleStart = this.state.configs.getOptionAsBool('DEFAULT_AUTO_START', 'upload_auto_send');
            var toggleClose = this.state.configs.getOptionAsBool('DEFAULT_AUTO_CLOSE', 'upload_auto_close');
            var toggleShowProcessed = this.state.configs.getOptionAsBool('UPLOAD_SHOW_PROCESSED', 'upload_show_processed', false);
            var overwriteType = this.state.configs.getOption('DEFAULT_EXISTING', 'upload_existing');

            return React.createElement(
                'div',
                { className: 'upload-options-pane react-mui-context' },
                React.createElement('span', { className: 'close-options mdi mdi-close', onClick: this.props.onDismiss }),
                React.createElement(
                    'div',
                    { className: 'option-row' },
                    maxUploadMessage
                ),
                React.createElement(
                    'div',
                    { className: 'option-row' },
                    React.createElement(ReactMUI.Toggle, { label: 'Start uploading automatically', labelPosition: 'right', toggled: toggleStart, defaultToggled: toggleStart, onToggle: this.updateField.bind(this, 'autostart') })
                ),
                React.createElement(
                    'div',
                    { className: 'option-row' },
                    React.createElement(ReactMUI.Toggle, { label: 'Close panel after upload is finished', labelPosition: 'right', toggled: toggleClose, onToggle: this.updateField.bind(this, 'autoclose') })
                ),
                React.createElement(
                    'div',
                    { className: 'option-row' },
                    React.createElement(ReactMUI.Toggle, { label: 'Show/hide processed files', labelPosition: 'right', toggled: toggleShowProcessed, onToggle: this.updateField.bind(this, 'show_processed') })
                ),
                React.createElement(
                    'div',
                    { className: 'option-row' },
                    React.createElement(
                        'div',
                        { style: { marginBottom: 10 } },
                        'If a file with the same name exists'
                    ),
                    React.createElement(
                        ReactMUI.RadioButtonGroup,
                        { ref: 'group', name: 'shipSpeed', defaultSelected: overwriteType, onChange: this.radioChange },
                        React.createElement(ReactMUI.RadioButton, { value: 'alert', label: 'Stop upload and alert me' }),
                        React.createElement(ReactMUI.RadioButton, { value: 'rename', label: 'Rename new file automatically' }),
                        React.createElement(ReactMUI.RadioButton, { value: 'overwrite', label: 'Overwrite existing file' })
                    )
                )
            );
        }

    });

    var ns = global.UploaderView || {};
    ns.DropUploader = DropUploader;
    global.UploaderView = ns;
})(window);
